@@ -15,12 +15,9 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::collections::HashMap;
 use std::fmt::Debug;
-use std::fmt::Formatter;
 use std::sync::Arc;
 
-use bytes::Buf;
 use http::Response;
 use http::StatusCode;
 use log::debug;
@@ -28,53 +25,37 @@ use reqsign::GoogleCredentialLoader;
 use reqsign::GoogleSigner;
 use reqsign::GoogleTokenLoad;
 use reqsign::GoogleTokenLoader;
-use serde::Deserialize;
-use serde_json;
 
+use super::GCS_SCHEME;
+use super::config::GcsConfig;
 use super::core::*;
-use super::delete::GcsDeleter;
+use super::deleter::GcsDeleter;
 use super::error::parse_error;
 use super::lister::GcsLister;
 use super::writer::GcsWriter;
 use super::writer::GcsWriters;
-use crate::raw::oio::BatchDeleter;
 use crate::raw::*;
-use crate::services::GcsConfig;
 use crate::*;
 
 const DEFAULT_GCS_ENDPOINT: &str = "https://storage.googleapis.com";
 const DEFAULT_GCS_SCOPE: &str = "https://www.googleapis.com/auth/devstorage.read_write";
 
-impl Configurator for GcsConfig {
-    type Builder = GcsBuilder;
-
-    #[allow(deprecated)]
-    fn into_builder(self) -> Self::Builder {
-        GcsBuilder {
-            config: self,
-            http_client: None,
-            customized_token_loader: None,
-        }
-    }
-}
-
 /// [Google Cloud Storage](https://cloud.google.com/storage) services support.
 #[doc = include_str!("docs.md")]
 #[derive(Default)]
 pub struct GcsBuilder {
-    config: GcsConfig,
+    pub(super) config: GcsConfig,
 
     #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
-    http_client: Option<HttpClient>,
-    customized_token_loader: Option<Box<dyn GoogleTokenLoad>>,
+    pub(super) http_client: Option<HttpClient>,
+    pub(super) customized_token_loader: Option<Box<dyn GoogleTokenLoad>>,
 }
 
 impl Debug for GcsBuilder {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        let mut ds = f.debug_struct("GcsBuilder");
-
-        ds.field("config", &self.config);
-        ds.finish_non_exhaustive()
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("GcsBuilder")
+            .field("config", &self.config)
+            .finish_non_exhaustive()
     }
 }
 
@@ -238,14 +219,13 @@ impl GcsBuilder {
 }
 
 impl Builder for GcsBuilder {
-    const SCHEME: Scheme = Scheme::Gcs;
     type Config = GcsConfig;
 
     fn build(self) -> Result<impl Access> {
-        debug!("backend build started: {:?}", self);
+        debug!("backend build started: {self:?}");
 
         let root = normalize_root(&self.config.root.unwrap_or_default());
-        debug!("backend use root {}", root);
+        debug!("backend use root {root}");
 
         // Handle endpoint and bucket name
         let bucket = match self.config.bucket.is_empty() {
@@ -253,7 +233,7 @@ impl Builder for GcsBuilder {
             true => Err(
                 Error::new(ErrorKind::ConfigInvalid, "The bucket is misconfigured")
                     .with_operation("Builder::build")
-                    .with_context("service", Scheme::Gcs),
+                    .with_context("service", GCS_SCHEME),
             ),
         }?;
 
@@ -312,21 +292,13 @@ impl Builder for GcsBuilder {
             core: Arc::new(GcsCore {
                 info: {
                     let am = AccessorInfo::default();
-                    am.set_scheme(Scheme::Gcs)
+                    am.set_scheme(GCS_SCHEME)
                         .set_root(&root)
                         .set_name(bucket)
                         .set_native_capability(Capability {
                             stat: true,
                             stat_with_if_match: true,
                             stat_with_if_none_match: true,
-                            stat_has_etag: true,
-                            stat_has_content_md5: true,
-                            stat_has_content_length: true,
-                            stat_has_content_type: true,
-                            stat_has_content_encoding: true,
-                            stat_has_last_modified: true,
-                            stat_has_user_metadata: true,
-                            stat_has_cache_control: true,
 
                             read: true,
 
@@ -363,11 +335,6 @@ impl Builder for GcsBuilder {
                             list_with_limit: true,
                             list_with_start_after: true,
                             list_with_recursive: true,
-                            list_has_etag: true,
-                            list_has_content_md5: true,
-                            list_has_content_length: true,
-                            list_has_content_type: true,
-                            list_has_last_modified: true,
 
                             presign: true,
                             presign_stat: true,
@@ -416,10 +383,6 @@ impl Access for GcsBackend {
     type Writer = GcsWriters;
     type Lister = oio::PageLister<GcsLister>;
     type Deleter = oio::BatchDeleter<GcsDeleter>;
-    type BlockingReader = ();
-    type BlockingWriter = ();
-    type BlockingLister = ();
-    type BlockingDeleter = ();
 
     fn info(&self) -> Arc<AccessorInfo> {
         self.core.info.clone()
@@ -433,37 +396,7 @@ impl Access for GcsBackend {
         }
 
         let slc = resp.into_body();
-
-        let meta: GetObjectJsonResponse =
-            serde_json::from_reader(slc.reader()).map_err(new_json_deserialize_error)?;
-
-        let mut m = Metadata::new(EntryMode::FILE);
-
-        m.set_etag(&meta.etag);
-        m.set_content_md5(&meta.md5_hash);
-
-        let size = meta
-            .size
-            .parse::<u64>()
-            .map_err(|e| Error::new(ErrorKind::Unexpected, "parse u64").set_source(e))?;
-        m.set_content_length(size);
-        if !meta.content_type.is_empty() {
-            m.set_content_type(&meta.content_type);
-        }
-
-        if !meta.content_encoding.is_empty() {
-            m.set_content_encoding(&meta.content_encoding);
-        }
-
-        if !meta.cache_control.is_empty() {
-            m.set_cache_control(&meta.cache_control);
-        }
-
-        m.set_last_modified(parse_datetime_from_rfc3339(&meta.updated)?);
-
-        if !meta.metadata.is_empty() {
-            m.with_user_metadata(meta.metadata);
-        }
+        let m = GcsCore::build_metadata_from_object_response(path, slc)?;
 
         Ok(RpStat::new(m))
     }
@@ -496,7 +429,7 @@ impl Access for GcsBackend {
     async fn delete(&self) -> Result<(RpDelete, Self::Deleter)> {
         Ok((
             RpDelete::default(),
-            BatchDeleter::new(GcsDeleter::new(self.core.clone())),
+            oio::BatchDeleter::new(GcsDeleter::new(self.core.clone())),
         ))
     }
 
@@ -549,87 +482,3 @@ impl Access for GcsBackend {
         )))
     }
 }
-
-/// The raw json response returned by [`get`](https://cloud.google.com/storage/docs/json_api/v1/objects/get)
-#[derive(Debug, Default, Deserialize)]
-#[serde(default, rename_all = "camelCase")]
-struct GetObjectJsonResponse {
-    /// GCS will return size in string.
-    ///
-    /// For example: `"size": "56535"`
-    size: String,
-    /// etag is not quoted.
-    ///
-    /// For example: `"etag": "CKWasoTgyPkCEAE="`
-    etag: String,
-    /// RFC3339 styled datetime string.
-    ///
-    /// For example: `"updated": "2022-08-15T11:33:34.866Z"`
-    updated: String,
-    /// Content md5 hash
-    ///
-    /// For example: `"md5Hash": "fHcEH1vPwA6eTPqxuasXcg=="`
-    md5_hash: String,
-    /// Content type of this object.
-    ///
-    /// For example: `"contentType": "image/png",`
-    content_type: String,
-    /// Content encoding of this object
-    ///
-    /// For example: "contentEncoding": "br"
-    content_encoding: String,
-    /// Cache-Control directive for the object data.
-    cache_control: String,
-    /// Custom metadata of this object.
-    ///
-    /// For example: `"metadata" : { "my-key": "my-value" }`
-    metadata: HashMap<String, String>,
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn test_deserialize_get_object_json_response() {
-        let content = r#"{
-  "kind": "storage#object",
-  "id": "example/1.png/1660563214863653",
-  "selfLink": "https://www.googleapis.com/storage/v1/b/example/o/1.png",
-  "mediaLink": "https://content-storage.googleapis.com/download/storage/v1/b/example/o/1.png?generation=1660563214863653&alt=media",
-  "name": "1.png",
-  "bucket": "example",
-  "generation": "1660563214863653",
-  "metageneration": "1",
-  "contentType": "image/png",
-  "contentEncoding": "br",
-  "cacheControl": "public, max-age=3600",
-  "storageClass": "STANDARD",
-  "size": "56535",
-  "md5Hash": "fHcEH1vPwA6eTPqxuasXcg==",
-  "crc32c": "j/un9g==",
-  "etag": "CKWasoTgyPkCEAE=",
-  "timeCreated": "2022-08-15T11:33:34.866Z",
-  "updated": "2022-08-15T11:33:34.866Z",
-  "timeStorageClassUpdated": "2022-08-15T11:33:34.866Z",
-  "metadata" : {
-    "location" : "everywhere"
-  }
-}"#;
-
-        let meta: GetObjectJsonResponse =
-            serde_json::from_str(content).expect("json Deserialize must succeed");
-
-        assert_eq!(meta.size, "56535");
-        assert_eq!(meta.updated, "2022-08-15T11:33:34.866Z");
-        assert_eq!(meta.md5_hash, "fHcEH1vPwA6eTPqxuasXcg==");
-        assert_eq!(meta.etag, "CKWasoTgyPkCEAE=");
-        assert_eq!(meta.content_type, "image/png");
-        assert_eq!(meta.content_encoding, "br".to_string());
-        assert_eq!(meta.cache_control, "public, max-age=3600".to_string());
-        assert_eq!(
-            meta.metadata,
-            HashMap::from_iter([("location".to_string(), "everywhere".to_string())])
-        );
-    }
-}
