@@ -15,23 +15,17 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#[cfg(feature = "services-ipmfs")]
+/// Default scheme for ipmfs service.
+pub const IPMFS_SCHEME: &str = "ipmfs";
+
 mod backend;
-#[cfg(feature = "services-ipmfs")]
+mod builder;
+mod config;
 mod core;
-#[cfg(feature = "services-ipmfs")]
-mod delete;
-#[cfg(feature = "services-ipmfs")]
+mod deleter;
 mod error;
-#[cfg(feature = "services-ipmfs")]
 mod lister;
-#[cfg(feature = "services-ipmfs")]
 mod writer;
 
-#[cfg(feature = "services-ipmfs")]
-mod builder;
-#[cfg(feature = "services-ipmfs")]
 pub use builder::IpmfsBuilder as Ipmfs;
-
-mod config;
 pub use config::IpmfsConfig;
